# Copyright 1999-2015. Parallels IP Holdings GmbH. All Rights Reserved.
import logging
import os
import sys
import xml.etree.ElementTree as ElementTree
from optparse import OptionParser

from parallels.operability_check import Severity
from OperabilityCheck import OperabilityCheck, RunError

_logger = logging.getLogger('operability_check')

def main():
    parser = OptionParser(description='Takes check commands from STDIN in XML format and returns result to STDOUT.')
    parser.add_option('-v', '--verbose', action='count', help='increase verbosity of logging messages (can be specified multiple times)')
    parser.add_option('--log-file', help='write logging messages to file specified')
    parser.add_option('--help-format', action='store_true', help="show input format")
    options, args = parser.parse_args()

    logLevel = logging.ERROR
    if 1 == options.verbose:
        logLevel = logging.INFO
    elif 2 <= options.verbose:
        logLevel = logging.DEBUG
    logging.basicConfig(
        level=logLevel
        , filename=options.log_file
        , format=u'[%(asctime)s|%(process)d] %(levelname)s: %(name)s: %(message)s'
    )

    if options.help_format:
        schemaPath = os.path.splitext(__file__)[0] + '.xsd'
        sys.stdout.write(open(schemaPath, 'r').read())
        sys.exit(0)

    try:
        input = ElementTree.parse(sys.stdin).getroot()
        output = ElementTree.Element('output')

        for commandInput in input:
            _logger.info(u'Run command %s', commandInput.tag)
            commandOutput = ElementTree.SubElement(output, commandInput.tag)
            try:
                commandName = 'Operability%s' % commandInput.tag.title().replace('-', '')
                commandModule = __import__(commandName)
                commandHandle = getattr(commandModule, commandName)(commandInput, commandOutput)
                commandHandle.run()
            except RunError as e:
                OperabilityCheck.addIssue(
                    Severity.INFO
                    , unicode(e)
                    , u'Check the service operability manually.'
                    , commandOutput
                )
            except Exception as e:
                _logger.debug(u"Run is failed:", exc_info=e)
                OperabilityCheck.addIssue(
                    Severity.INFO
                    , u'Unable to check the service by command \'%s\': %s' % (commandInput.tag, unicode(e))
                    , u'Check the service operability manually.'
                    , commandOutput
                )

        ElementTree.ElementTree(output).write(sys.stdout, 'UTF-8')
    except Exception as e:
        sys.stderr.write('%s\n' % e)
        sys.exit(1)

if __name__ == '__main__':
    main()
